/*
    ssystem 1.5
    Copyright (C) 1997-1998  Raul Alonso <amil@las.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <GL/glut.h>
#ifndef PLANET
#include "cfgparse.tab.h"
#endif


#define NUMSTARS 3141 /* 3141 for full star catalog, decrease number for better
			performance, first N brighter stars are used */

#define MAXBODIES 50

#ifndef PI
#define PI 3.14159265358979323846
#define PIl 3.14159265358979323846L
#endif

#define DATEOFELEMENTS 2450680.5 /* In Julian days */

/* Macros used to scale down distances/radii so we can reduce the
  jerkiness effect in outter bodies */
#define DFACTOR 1000.0
#define RADIUSSCALE(x) ((x)*0.1/DFACTOR)
#define DISTCORRECTION(x) ((x)*149.0/DFACTOR)

/* Need no comment ;-) */
#define DEG2RAD(x) ((x)*PI/180.0)
#define RAD2DEG(x) ((x)*180.0/PI)
#define DISTANCE(x,y,z) sqrt((x)*(x)+(y)*(y)+(z)*(z))

#ifndef WIN32
#define error(s) fprintf(stderr,s)
#define scmp strcasecmp
#else
#define error(s) MessageBox(NULL,s,"ERROR", MB_OK); 
#define scmp _stricmp
#endif

extern GLfloat LightPos[4];
extern GLfloat ambient[4];
extern GLfloat White[4];
extern GLfloat Black[4];
extern int texture,smodel,lighting,drawstars;
extern int Stars,Ecliptic,red,polaris;
extern int ImgWidth, ImgHeight;
extern double days,timefactor;
extern char texturepath[100];
extern int slices,stacks,NUMBODIES;

typedef struct {
	char Name[20];
	int Type;
	int Sat,VSOP87;
	GLUquadricObj *Object;
	GLubyte *Image;
	GLuint Sphere,Trail,*textures,texnum,texwidth,texheight;
	float DeltaRotation,Radius,xflat,yflat,zflat,Rotation,Degrees,OrbitalPeriod;
	float Inclination,AscendingNode,Perihelion,MeanDistance,DailyMotion;
	float Eccentricity,MeanLongitude,Color[4];
	double posx,posy,posz;
} planetdata;

typedef struct {
	char Name[20];
	double posx,posy,posz;
	float magnitude,color[3];
	char type;
} stardata;

extern planetdata planets[MAXBODIES];
extern stardata stars[NUMSTARS];	
	

void Init(void),InitStars(float, int);
void UpdatePositions(double, int);
void joystick(void);
void joydetect(void);
void ReadConfigFile(void);
double JulianDay(int Y, int M, int D);
void EarthPos(double JD, double *Long, double *Lat, double *Rad);
void JupiterPos(double JD, double *Long, double *Lat, double *Rad);
void MarsPos(double JD, double *Long, double *Lat, double *Rad);
void MercuryPos(double JD, double *Long, double *Lat, double *Rad);
void NeptunePos(double JD, double *Long, double *Lat, double *Rad);
void SaturnPos(double JD, double *Long, double *Lat, double *Rad);
void UranusPos(double JD, double *Long, double *Lat, double *Rad);
void VenusPos(double JD, double *Long, double *Lat, double *Rad);
double ReadTerms(double T, const char *filename);
